@@ -191,7 +191,6 @@ add_cal_esource (EAccount *account, GSList *folders, ExchangeMAPIFolderType fold
 	GSList *ids, *temp_list, *old_sources = NULL;
 	gchar *base_uri = NULL;
 	gboolean is_new_group = FALSE;
-	
 
 	if (folder_type == MAPI_FOLDER_TYPE_APPOINTMENT) {
 		conf_key = CALENDAR_SOURCES;
@@ -307,7 +306,7 @@ add_cal_esource (EAccount *account, GSList *folders, ExchangeMAPIFolderType fold
 		/* these were not found on the server by fid, thus remove them */
 		for (temp_list = old_sources; temp_list; temp_list = temp_list->next) {
 			ESource *source = temp_list->data;
-		
+
 			if (source && E_IS_SOURCE (source)) {
 				if (strcmp (e_source_get_property (source, "public"), "yes") != 0)
 					e_source_group_remove_source (group, source);
@@ -338,15 +337,15 @@ void exchange_mapi_add_esource (CamelURL *url, const gchar *folder_name, const g
 	ESource *source = NULL;
 	gchar *relative_uri = NULL;
 	gchar *base_uri = NULL;
-	
+
 	if (url == NULL)
 		return;
-	
-	if (folder_type == MAPI_FOLDER_TYPE_APPOINTMENT) 
+
+	if (folder_type == MAPI_FOLDER_TYPE_APPOINTMENT)
 		conf_key = CALENDAR_SOURCES;
-	else if (folder_type == MAPI_FOLDER_TYPE_TASK) 
+	else if (folder_type == MAPI_FOLDER_TYPE_TASK)
 		conf_key = TASK_SOURCES;
-	else if (folder_type == MAPI_FOLDER_TYPE_MEMO) 
+	else if (folder_type == MAPI_FOLDER_TYPE_MEMO)
 		conf_key = JOURNAL_SOURCES;
 	else if (folder_type == MAPI_FOLDER_TYPE_JOURNAL)
 		conf_key = JOURNAL_SOURCES;
@@ -361,21 +360,20 @@ void exchange_mapi_add_esource (CamelURL *url, const gchar *folder_name, const g
 	source_list = e_source_list_new_for_gconf (client, conf_key);
 	base_uri = g_strdup_printf ("%s%s@%s/", MAPI_URI_PREFIX, url->user, url->host);
 	group = e_source_list_peek_group_by_base_uri (source_list, base_uri);
-	sources = e_source_group_peek_sources (group);	
+	sources = e_source_group_peek_sources (group);
 	for (; sources != NULL; sources = g_slist_next (sources)) {
 		ESource *source = E_SOURCE (sources->data);
-		gchar* folder_id = e_source_get_duped_property (source, "folder-id");
-		if (folder_id && fid) { 
-			if (strcmp (fid, folder_id) != 0) 
+		gchar * folder_id = e_source_get_duped_property (source, "folder-id");
+		if (folder_id && fid) {
+			if (strcmp (fid, folder_id) != 0)
 				continue;
 			else {
 				g_warning ("%s: %s: Esource Already exist \n", G_STRLOC, G_STRFUNC);
-				return;	
+				return;
 			}
 		}
 	}
 
-	
 	relative_uri = g_strconcat (";", fid, NULL);
 	source = e_source_new (folder_name, relative_uri);
 	e_source_set_property (source, "auth", "1");
@@ -395,7 +393,7 @@ void exchange_mapi_add_esource (CamelURL *url, const gchar *folder_name, const g
 
 	g_object_unref (source);
 	g_free (relative_uri);
-			
+
 	if (!e_source_list_add_group (source_list, group, -1))
 		return;
 
@@ -407,8 +405,7 @@ void exchange_mapi_add_esource (CamelURL *url, const gchar *folder_name, const g
 	g_object_unref (client);
 }
 
-
-void exchange_mapi_remove_esource (CamelURL *url, const gchar* folder_name, const gchar *fid, gint folder_type)
+void exchange_mapi_remove_esource (CamelURL *url, const gchar * folder_name, const gchar *fid, gint folder_type)
 {
 	ESourceList *source_list = NULL;
 	ESourceGroup *group = NULL;
@@ -416,15 +413,15 @@ void exchange_mapi_remove_esource (CamelURL *url, const gchar* folder_name, cons
 	GConfClient* client;
 	GSList *sources=NULL;
 	gchar *base_uri = NULL;
-	
+
 	if (url == NULL)
 		return;
 
-	if (folder_type == MAPI_FOLDER_TYPE_APPOINTMENT) 
+	if (folder_type == MAPI_FOLDER_TYPE_APPOINTMENT)
 		conf_key = CALENDAR_SOURCES;
-	else if (folder_type == MAPI_FOLDER_TYPE_TASK) 
+	else if (folder_type == MAPI_FOLDER_TYPE_TASK)
 		conf_key = TASK_SOURCES;
-	else if (folder_type == MAPI_FOLDER_TYPE_MEMO) 
+	else if (folder_type == MAPI_FOLDER_TYPE_MEMO)
 		conf_key = JOURNAL_SOURCES;
 	else if (folder_type == MAPI_FOLDER_TYPE_JOURNAL)
 		conf_key = JOURNAL_SOURCES;
@@ -439,11 +436,11 @@ void exchange_mapi_remove_esource (CamelURL *url, const gchar* folder_name, cons
 	source_list = e_source_list_new_for_gconf (client, conf_key);
 	base_uri = g_strdup_printf ("%s%s@%s/", MAPI_URI_PREFIX, url->user, url->host);
 	group = e_source_list_peek_group_by_base_uri (source_list, base_uri);
-	sources = e_source_group_peek_sources (group);	
-	
+	sources = e_source_group_peek_sources (group);
+
 	for (; sources != NULL; sources = g_slist_next (sources)) {
 		ESource *source = E_SOURCE (sources->data);
-		gchar* folder_id = e_source_get_duped_property (source, "folder-id"); 
+		gchar * folder_id = e_source_get_duped_property (source, "folder-id");
 		if (folder_id && fid)
 			if (strcmp(fid, folder_id) == 0) {
 				e_source_group_remove_source(group, source);
@@ -457,7 +454,6 @@ void exchange_mapi_remove_esource (CamelURL *url, const gchar* folder_name, cons
 
 }
 
-
 static void
 remove_cal_esource (EAccount *existing_account_info, ExchangeMAPIFolderType folder_type, CamelURL *url)
 {
@@ -783,7 +779,7 @@ update_sources_idle_cb (gpointer data)
 	folders_list = exchange_mapi_connection_peek_folders_list (conn);
 
 	if (account->enabled && lookup_account_info (account->uid)) {
-		mapi_id_t trash_fid = exchange_mapi_connection_get_default_folder_id (conn, olFolderDeletedItems);
+		mapi_id_t trash_fid = exchange_mapi_connection_get_default_folder_id (conn, olFolderDeletedItems, NULL);
 
 		add_addressbook_sources (account, folders_list, trash_fid);
 		add_calendar_sources (account, folders_list, trash_fid);
@@ -949,10 +945,17 @@ mapi_account_removed (EAccountList *account_listener, EAccount *account)
 	if (url != NULL) {
 		const gchar *profile = camel_url_get_param (url, "profile");
 		gchar *key = camel_url_to_string (url, CAMEL_URL_HIDE_PASSWORD | CAMEL_URL_HIDE_PARAMS);
-		exchange_mapi_delete_profile (profile);
+		GError *error = NULL;
+
+		exchange_mapi_delete_profile (profile, &error);
 		e_passwords_forget_password (EXCHANGE_MAPI_PASSWORD_COMPONENT, key);
+
 		g_free (key);
 		camel_url_free (url);
+		if (error) {
+			g_warning ("%s: Failed to delete profile: %s", G_STRFUNC, error->message);
+			g_error_free (error);
+		}
 	}
 
 	/* Free up the structure */
@@ -986,9 +989,10 @@ create_profile_entry (CamelURL *url, EAccount *account)
 		g_free (key);
 
 		if (password) {
+			GError *error = NULL;
 			guint32 cp_flags = (camel_url_get_param (url, "ssl") && g_str_equal (camel_url_get_param (url, "ssl"), "1")) ? CREATE_PROFILE_FLAG_USE_SSL : CREATE_PROFILE_FLAG_NONE;
 
-			status = exchange_mapi_create_profile (url->user, password, camel_url_get_param (url, "domain"), url->host, cp_flags, NULL, NULL, NULL);
+			status = exchange_mapi_create_profile (url->user, password, camel_url_get_param (url, "domain"), url->host, cp_flags, NULL, NULL, &error);
 			if (status) {
 				/* profile was created, try to connect to the server */
 				ExchangeMapiConnection *conn;
@@ -997,7 +1001,7 @@ create_profile_entry (CamelURL *url, EAccount *account)
 				status = FALSE;
 				profname = exchange_mapi_util_profile_name (url->user, camel_url_get_param (url, "domain"), url->host, FALSE);
 
-				conn = exchange_mapi_connection_new (profname, password);
+				conn = exchange_mapi_connection_new (profname, password, &error);
 				if (conn) {
 					status = exchange_mapi_connection_connected (conn);
 					g_object_unref (conn);
@@ -1005,6 +1009,11 @@ create_profile_entry (CamelURL *url, EAccount *account)
 
 				g_free (profname);
 			}
+
+			if (error) {
+				g_warning ("%s: Failed to create profile: %s", G_STRFUNC, error->message);
+				g_error_free (error);
+			}
 		}
 
 		++attempts;
